@@ -2,18 +2,16 @@ import logging
 import os
 import sys
 
-import regex
-
 from . import plugins
 from . import utils
 from .plugin import get_plugins, load_plugins_by_package, DatasetPlugin, WritingPlugin, ValidationPlugin, SchemaPlugin, \
     FileHandlerPlugin
-from .plugins.plugin_query import BoolExpr, Select, EqExpr, AnyExpr, AllExpr, ReExpr, CustomOpExpr, \
+from .query import BoolExpr, Select, EqExpr, AnyExpr, AllExpr, ReExpr, CustomOpExpr, \
     EntityExpr
 
 LOGGER = logging.getLogger("ancpbids")
 
-ENTITIES_PATTERN = regex.compile(r'(([^\W_]+)-([^\W_]+)_)+([^\W_]+)((\.[^\W_]+)+)')
+# ENTITIES_PATTERN = regex.compile(r'(([^\W_]+)-([^\W_]+)_)+([^\W_]+)((\.[^\W_]+)+)')
 
 
 def load_dataset(base_dir: str):
@@ -35,6 +33,8 @@ def load_dataset(base_dir: str):
     str
         an object instance of type :py:class:`ancpbids.model.Dataset` which represents the dataset as an in-memory graph
     """
+    if not os.path.isdir(base_dir):
+        raise ValueError("Invalid Directory")
     schema = load_schema(base_dir)
     ds = schema.Dataset()
     ds.name = os.path.basename(base_dir)
@@ -73,9 +73,8 @@ def load_schema(base_dir):
             if schema_name in sys.modules:
                 schema = sys.modules[schema_name]
                 return schema
-    # assume using the earliest supported schema
-    from . import model_v1_7_0
-    return model_v1_7_0
+    # assume using the latest supported schema
+    return model_latest
 
 
 def save_dataset(ds: object, target_dir: str, context_folder=None):
@@ -153,10 +152,11 @@ def write_derivative(ds, derivative):
 load_plugins_by_package(plugins, ranking=0, system=True)
 
 # execute all SchemaPlugins, these plugins may monkey-patch the schema
-from ancpbids import model_v1_7_0, model_v1_7_1
+from ancpbids import model_v1_8_0
+from ancpbids import model_v1_8_0 as model_latest
 
 for pl in get_plugins(SchemaPlugin):
-    for model in [model_v1_7_0, model_v1_7_1]:
+    for model in [model_latest]:
         pl.execute(model)
 
 # load file handler plugins
