@@ -1,12 +1,9 @@
-from ..plugin import DatasetPlugin
-
 import inspect
 import os
+import re
 
-import regex
-
-from ancpbids import ENTITIES_PATTERN
 from .. import utils
+from ..plugin import DatasetPlugin
 
 
 class DatasetPopulationPlugin(DatasetPlugin):
@@ -44,26 +41,25 @@ class DatasetPopulationPlugin(DatasetPlugin):
             self._convert_files_to_artifacts(folder)
 
     def _convert_to_artifact(self, file):
-        match = ENTITIES_PATTERN.match(file.name)
-        if not match:
+        parts = utils.parse_bids_name(file.name)
+        if not parts:
             return None
         artifact = self.schema.Artifact()
         artifact.name = file.name
-        for pair in zip(match.captures(2), match.captures(3)):
+        for key, value in parts['entities'].items():
             entity = self.schema.EntityRef()
-            key = pair[0]
             entity.key = key
-            value = self.schema.process_entity_value(key, pair[1])
+            value = self.schema.process_entity_value(key, value)
             entity.value = value
             artifact.entities.append(entity)
-        artifact.suffix = match[4]
-        artifact.extension = match[5]
+        artifact.suffix = parts['suffix']
+        artifact.extension = parts['extension']
         return artifact
 
     def _handle_direct_folders(self, parent, member, pattern, new_type):
         if not isinstance(parent, self.schema.Folder):
             return
-        folders = list(filter(lambda f: regex.match(pattern, f.name), parent.get_folders_sorted()))
+        folders = list(filter(lambda f: re.match(pattern, f.name), parent.get_folders_sorted()))
         for folder in folders:
             obj = new_type()
             obj.name = folder.name
@@ -177,7 +173,7 @@ class DatasetPopulationPlugin(DatasetPlugin):
 
     def _map_object(self, model_type, json_object):
         target = model_type()
-        members = self.schema.get_members(model_type, False)
+        members = self.schema.get_members(model_type, True)
         actual_props = json_object.keys()
         direct_props = list(map(lambda m: (m['name'], m), members))
         for prop_name, prop in direct_props:
