@@ -1,9 +1,13 @@
 import fnmatch
 import inspect
+import math
 import os
+import sys
 from difflib import SequenceMatcher
 
 from ancpbids.plugin import SchemaPlugin
+from ancpbids.query import Select, query, query_entities
+from ancpbids.utils import resolve_segments, convert_to_relative
 
 
 def has_entity(artifact, entity_):
@@ -28,10 +32,10 @@ def add_entity(artifact, key, value):
     artifact.entities.append(eref)
 
 
-def load_file_contents(folder, file_name):
+def load_file_contents(folder, file_name, return_type: str = None):
     from ancpbids import utils
     file_path = get_absolute_path(folder, file_name)
-    contents = utils.load_contents(file_path)
+    contents = utils.load_contents(file_path, return_type)
     return contents
 
 
@@ -100,7 +104,7 @@ def create_folder(folder, type_=None, **kwargs):
     return sub_folder
 
 
-def create_derivative(ds, **kwargs):
+def create_derivative(ds, path=None, **kwargs):
     schema = ds.get_schema()
     derivatives_folder = ds.derivatives
     if not ds.derivatives:
@@ -108,9 +112,11 @@ def create_derivative(ds, **kwargs):
         derivatives_folder.parent_object_ = ds
         derivatives_folder.name = "derivatives"
         ds.derivatives = derivatives_folder
+    path = convert_to_relative(ds, path)
+    target_folder, _ = resolve_segments(derivatives_folder, path, create_if_missing=True)
     derivative = schema.DerivativeFolder(**kwargs)
-    derivative.parent_object_ = derivatives_folder
-    derivatives_folder.folders.append(derivative)
+    derivative.parent_object_ = target_folder
+    target_folder.folders.append(derivative)
 
     derivative.dataset_description = schema.DerivativeDatasetDescriptionFile()
     derivative.dataset_description.parent_object_ = derivative
@@ -122,16 +128,29 @@ def create_derivative(ds, **kwargs):
     return derivative
 
 
-def get_file(folder, file_name, from_meta=True):
-    file = next(filter(lambda file: file.name == file_name, folder.files), None)
-    if not file and from_meta:
-        # search in metadatafiles
-        file = next(filter(lambda file: file.name == file_name, folder.metadatafiles), None)
+def create_dataset(schema, **kwargs):
+    ds = schema.Dataset()
+    ds.update(**kwargs)
+    ds.dataset_description = schema.DatasetDescriptionFile(name="dataset_description.json")
+    ds.dataset_description.BIDSVersion = schema.VERSION
+    ds.dataset_description.parent_object_ = ds
+    return ds
+
+
+def get_file(folder, file_name):
+    folder, file_name = resolve_segments(folder, file_name, True)
+    if not folder:
+        return None
+    schema = folder.get_schema()
+    direct_files = folder.to_generator(depth_first=True, depth=1, filter_=lambda n: isinstance(n, schema.File))
+    file = next(filter(lambda f: f.name == file_name, direct_files), None)
     return file
 
 
 def get_files(folder, name_pattern):
-    return list(filter(lambda file: fnmatch.fnmatch(file.name, name_pattern), folder.files))
+    schema = folder.get_schema()
+    direct_files = folder.to_generator(depth_first=True, depth=1, filter_=lambda n: isinstance(n, schema.File))
+    return list(filter(lambda file: fnmatch.fnmatch(file.name, name_pattern), direct_files))
 
 
 def remove_folder(folder, folder_name):
@@ -139,7 +158,9 @@ def remove_folder(folder, folder_name):
 
 
 def get_folder(folder, folder_name):
-    return next(filter(lambda f: f.name == folder_name, folder.folders), None)
+    schema = folder.get_schema()
+    direct_folders = folder.to_generator(depth_first=True, depth=1, filter_=lambda n: isinstance(n, schema.Folder))
+    return next(filter(lambda f: f.name == folder_name, direct_folders), None)
 
 
 def get_files_sorted(folder):
@@ -150,20 +171,23 @@ def get_folders_sorted(folder):
     return sorted(folder.folders, key=lambda f: f.name)
 
 
-def to_generator(source, depth_first=False, filter_=None):
-    schema = source.get_schema()
+def to_generator(source, depth_first=False, filter_=None, depth=1000):
+    if depth < 0:
+        return
+
     if not depth_first:
         if filter_ and not filter_(source):
             return
         yield source
 
+    schema = source.get_schema()
     for key, value in source.items():
         if isinstance(value, schema.Model):
-            yield from to_generator(value, depth_first, filter_)
+            yield from to_generator(value, depth_first, filter_, depth - 1)
         elif isinstance(value, list):
             for item in value:
                 if isinstance(item, schema.Model):
-                    yield from to_generator(item, depth_first, filter_)
+                    yield from to_generator(item, depth_first, filter_, depth - 1)
 
     if depth_first:
         if filter_ and not filter_(source):
@@ -211,7 +235,10 @@ def get_members(schema, element_type, include_superclass=True):
         superclass = element_type
         while True:
             try:
-                superclass = inspect.getmro(superclass)[1]
+                mro = inspect.getmro(superclass)
+                if len(mro) < 2:
+                    break
+                superclass = mro[1]
                 if not superclass or superclass == schema.Model:
                     break
                 super_members = super_members + _get_element_members(schema, superclass)
@@ -234,7 +261,8 @@ def _to_type(schema, model_type_name: str):
 def _trim_int(value):
     try:
         # remove paddings/fillers in index values: 001 -> 1, 000230 -> 230
-        return str(int(value))
+        # TODO return PaddedInt as done by PyBIDS
+        return int(value)
     except ValueError:
         return value
 
@@ -242,34 +270,57 @@ def _trim_int(value):
 def process_entity_value(schema, key, value):
     if not value:
         return value
+    if isinstance(key, schema.EntityEnum):
+        key = key.literal_
     for sc_entity in filter(lambda e: e.literal_ == key, schema.EntityEnum.__members__.values()):
         if sc_entity.format_ == 'index':
             if isinstance(value, list):
-                return list(map(lambda v: _trim_int(v), value))
+                return list(map(lambda v: _trim_int(v) if v is not None else v, value))
             else:
                 return _trim_int(value)
     return value
 
 
 def fuzzy_match_entity_key(schema, user_key):
-    return fuzzy_match_entity(schema, user_key).entity_
+    return fuzzy_match_entity(schema, user_key).literal_
 
 
 def fuzzy_match_entity(schema, user_key):
     ratios = list(
         map(lambda item: (
             item,
-            1.0 if item.literal_.startswith(user_key) else SequenceMatcher(None, user_key,
-                                                                           item.literal_).quick_ratio()),
+            1.0 if item.name.startswith(user_key) else SequenceMatcher(None, user_key,
+                                                                           item.name).quick_ratio()),
             list(schema.EntityEnum)))
     ratios = sorted(ratios, key=lambda t: t[1])
     return ratios[-1][0]
 
 
+def get_parent(file_or_folder):
+    if hasattr(file_or_folder, 'parent_object_'):
+        return file_or_folder.parent_object_
+    return None
+
+
+def select(context, target_type):
+    return Select(context, target_type)
+
+
+def get_entities(artifact):
+    return {e['key']: e['value'] for e in artifact.entities}
+
+
 class PatchingSchemaPlugin(SchemaPlugin):
     def execute(self, schema):
+        schema.Model.__hash__ = lambda self: hash(tuple(self))
+        schema.Folder.select = select
+        schema.Folder.query = query
+        schema.Folder.query_entities = query_entities
+        schema.File.get_parent = get_parent
+        schema.Folder.get_parent = get_parent
         schema.Artifact.has_entity = has_entity
         schema.Artifact.get_entity = get_entity
+        schema.Artifact.get_entities = get_entities
         schema.Artifact.add_entity = add_entity
         schema.Folder.load_file_contents = load_file_contents
         schema.File.load_contents = load_contents
@@ -297,3 +348,5 @@ class PatchingSchemaPlugin(SchemaPlugin):
         schema.process_entity_value = lambda key, value: process_entity_value(schema, key, value)
         schema.fuzzy_match_entity_key = lambda user_key: fuzzy_match_entity_key(schema, user_key)
         schema.fuzzy_match_entity = lambda user_key: fuzzy_match_entity(schema, user_key)
+
+        schema.create_dataset = lambda **kwargs: create_dataset(schema, **kwargs)
