@@ -7,13 +7,70 @@ FILE_WRITERS = {}
 LOGGER = logging.getLogger(__file__)
 
 
-def load_contents(file_path):
+def parse_bids_name(name: str):
+    """Parses a given string (file name) according to the BIDS naming scheme.
+
+    Parameters
+    ----------
+    name
+        The file name to parse. If a full path (with path separaters), the path segments will be ignored.
+
+    Returns
+    -------
+    dict
+        A dictionary describing the BIDS naming components.
+
+    Examples
+    --------
+
+    >>> bids_obj = parse_bids_name("sub-11_task-mixedgamblestask_run-02_bold.nii.gz")
+    {'entities': {'sub': '11', 'task': 'mixedgamblestask', 'run': '02'}, 'suffix': 'bold', 'extension': '.nii.gz'}
+
+    """
+    base_name = os.path.basename(name)
+    parts = base_name.split(os.extsep, 1)
+    if len(parts) != 2:
+        # if extension missing, then not a valid BIDS file
+        return None
+
+    extension = os.extsep + parts[1]
+    underscore_parts = parts[0].split('_')
+
+    if len(underscore_parts) < 2:
+        return None
+
+    # last segment must be suffix
+    suffix = underscore_parts[-1]
+    if '-' in suffix:
+        return None
+
+    entities = {}
+    for i in range(0, len(underscore_parts) - 1):
+        dash_parts = underscore_parts[i].split('-')
+        if len(dash_parts) < 2:
+            # not a key-value pair
+            return None
+        entities[dash_parts[0]] = dash_parts[1]
+
+    return {
+        'entities': entities,
+        'suffix': suffix,
+        'extension': extension
+    }
+
+
+def load_contents(file_path, return_type: str = None):
     """Loads the contents of the provided file path.
 
     Parameters
     ----------
     file_path :
         the file path to load contents from
+    return_type:
+        A hint to consider when deciding how to load the contents of the provided file.
+        For example, to load a TSV file as a pandas DataFrame the return_type should be 'dataframe',
+        to load a numpy ndarray, the return_type should be 'ndarray'.
+        It is up to the registered file handlers to correctly interpret the return_type.
 
     Returns
     -------
@@ -35,7 +92,7 @@ def load_contents(file_path):
         reader = FILE_READERS['txt']
     if reader is None:
         raise ValueError('No file reader registered to load file %s' % file_path)
-    return reader(file_path)
+    return reader(file_path, return_type=return_type)
 
 
 def write_contents(file_path: str, contents):
@@ -140,3 +197,33 @@ def fetch_dataset(dataset_id: str, output_dir='~/.ancp-bids/datasets'):
     z = zipfile.ZipFile(download_path)
     z.extractall(output_dir)
     return output_path
+
+
+def resolve_segments(root_folder, path_, last_seg_file=False, create_if_missing=False):
+    context = root_folder
+    if path_:
+        normalized_path = os.path.normpath(path_)
+        path_segments = normalized_path.split(os.sep)
+        path_segments = list(filter(lambda s: s != '', path_segments))
+        if last_seg_file:
+            path_ = path_segments[-1]
+            path_segments = path_segments[:-1]
+
+        for seg in path_segments:
+            context = context.get_folder(seg)
+            if not context:
+                if create_if_missing:
+                    context.create_folder(seg)
+                else:
+                    break
+    return context, path_
+
+
+def convert_to_relative(dataset, path):
+    if path and path.startswith(dataset.base_dir_):
+        path = os.path.normpath(path)
+        path = path[len(dataset.base_dir_):]
+        path_segments = path.split(os.sep)
+        path_segments = list(filter(lambda s: s != '', path_segments))
+        path = os.path.join(*path_segments)
+    return path
