@@ -1,18 +1,28 @@
 import numpy
 
-from ancpbids import load_dataset, ENTITIES_PATTERN
+from ancpbids import load_dataset
+from ancpbids.utils import parse_bids_name
 from ..base_test_case import *
 
 
 class BasicTestCase(BaseTestCase):
     def test_naming_scheme(self):
-        valid_names = ["sub-11_task-mixedgamblestask_run-02_events.tsv", "sub-11_dwi.nii.gz", "x-01_y-02_z-03_xyz.abc", "sub-01_task-mixedgamblestask_run-01_bold.nii.gz"]
+        valid_names = ["sub-11_task-mixedgamblestask_run-02_events.tsv", "sub-11_dwi.nii.gz", "x-01_y-02_z-03_xyz.abc",
+                       "sub-01_task-mixedgamblestask_run-01_bold.nii.gz"]
         for name in valid_names:
-            self.assertTrue(ENTITIES_PATTERN.match(name))
+            self.assertTrue(parse_bids_name(name) is not None)
 
         invalid_names = ["sub-04_T1w_bias.nii.gz", "cat.jpg", "readme.txt"]
         for name in invalid_names:
-            self.assertFalse(ENTITIES_PATTERN.match(name))
+            self.assertTrue(parse_bids_name(name) is None)
+
+    def test_parse_bids_name(self):
+        bids_obj = parse_bids_name("sub-11_task-mixedgamblestask_run-02_bold.nii.gz")
+        self.assertTrue(isinstance(bids_obj, dict))
+        self.assertEqual(['sub', 'task', 'run'], list(bids_obj['entities'].keys()))
+        self.assertEqual(['11', 'mixedgamblestask', '02'], list(bids_obj['entities'].values()))
+        self.assertEqual('bold', bids_obj['suffix'])
+        self.assertEqual('.nii.gz', bids_obj['extension'])
 
     def test_ds005_basic_structure(self):
         ds005 = load_dataset(DS005_DIR)
@@ -70,9 +80,14 @@ class BasicTestCase(BaseTestCase):
     def test_tsv_file_contents(self):
         ds005 = load_dataset(DS005_DIR)
         participants = ds005.load_file_contents("participants.tsv")
-        self.assertTrue(isinstance(participants, numpy.ndarray), "Expected a numpy.ndarray")
+        self.assertListEqual(['participant_id', 'sex', 'age'], list(participants[0].keys()))
+        self.assertEqual(16, len(participants))
+        participants = ds005.load_file_contents("participants.tsv", return_type="ndarray")
         self.assertListEqual(['participant_id', 'sex', 'age'], list(participants.dtype.names))
-        self.assertEqual(16, participants.shape[0])
+        self.assertEqual(16, len(participants))
+        participants = ds005.load_file_contents("participants.tsv", return_type="dataframe")
+        self.assertListEqual(['participant_id', 'sex', 'age'], list(participants.columns))
+        self.assertEqual(16, len(participants))
 
     def test_parse_entities_in_filenames(self):
         ds005 = load_dataset(DS005_DIR)
@@ -98,7 +113,34 @@ class BasicTestCase(BaseTestCase):
 
         entity = entities[2]
         self.assertEqual("run", entity.key)
-        self.assertEqual("1", entity.value)
+        self.assertEqual(1, entity.value)
+
+    def test_to_generator(self):
+        ds005 = load_dataset(DS005_DIR)
+        schema = ds005.get_schema()
+
+        all_direct_files = list(
+            ds005.to_generator(depth_first=True, depth=1, filter_=lambda n: isinstance(n, schema.File)))
+        self.assertEqual(8, len(all_direct_files))
+
+    def test_get_files_and_folders(self):
+        ds005 = load_dataset(DS005_DIR)
+
+        file = ds005.get_file("dataset_description.json")
+        self.assertIsNotNone(file)
+        self.assertEqual("dataset_description.json", file.name)
+
+        file = ds005.get_file("sub-01/func/sub-01_task-mixedgamblestask_run-01_bold.nii.gz")
+        self.assertIsNotNone(file)
+        self.assertEqual("sub-01_task-mixedgamblestask_run-01_bold.nii.gz", file.name)
+
+    def test_repr(self):
+        ds005 = load_dataset(DS005_DIR)
+        self.assertEqual("{'name': 'ds005'}", str(ds005))
+        self.assertEqual("{'name': 'derivatives'}", str(ds005.derivatives))
+        self.assertEqual("{'name': 'README'}", str(ds005.README))
+        expected = "{'name': 'dataset_description.json', 'Name': 'Mixed-gambles task', 'BIDSVersion': '1.0.0rc2', 'License': 'This dataset is made available u[...]'}"
+        self.assertEqual(expected, str(ds005.dataset_description))
 
 
 if __name__ == '__main__':
