@@ -73,7 +73,7 @@ class QueryTestCase(BaseTestCase):
         sorted_entities = layout.get_entities(scope='raw', sort=True)
         # note: 'ds' and 'type' entities are contained in folder 'models' at dataset level, so considered raw data
         self.assertListEqual(['ds', 'run', 'sub', 'task', 'type'], list(sorted_entities.keys()))
-        self.assertListEqual(['1', '2', '3'], sorted_entities['run'])
+        self.assertListEqual([1, 2, 3], sorted_entities['run'])
         self.assertEqual(['%02d' % i for i in range(1, 17)], sorted_entities['sub'])
         self.assertListEqual(['mixedgamblestask'], sorted_entities['task'])
 
@@ -89,31 +89,18 @@ class QueryTestCase(BaseTestCase):
 
     def test_bidslayout_get_metadata(self):
         layout = ancpbids.BIDSLayout(DS005_DIR)
-        metadata = layout.get_metadata(task='mixedgamblestask', suffix='bold')
+        metadata = layout.get_metadata("sub-01/func/sub-01_task-mixedgamblestask_run-01_bold.nii.gz")
         self.assertTrue(isinstance(metadata, dict))
         self.assertEqual(2.0, metadata['RepetitionTime'])
         self.assertEqual('mixed-gambles task', metadata['TaskName'])
         self.assertListEqual([0.0, 0.0571, 0.1143, 0.1714, 0.2286, 0.2857], metadata['SliceTiming'])
 
-    def test_bidslayout_get_metadata_inheritance(self):
-        layout = ancpbids.BIDSLayout(DS005_DIR + "-small")
-        task_bold = layout.dataset.get_file('task-mixedgamblestask_bold.json').load_contents()
-        # at the top level, RepetionTime is set to 2.0
-        self.assertEqual(2.0, task_bold['RepetitionTime'])
-
-        # now load all metadata matching the filter
-        metadata = layout.get_metadata(task='mixedgamblestask', suffix='bold')
-        self.assertTrue(isinstance(metadata, dict))
-
-        # now, since at subject/run level the RepetionTime is overridden, it should be 2.5
-        self.assertEqual(2.5, metadata['RepetitionTime'])
-
     def test_query_language(self):
         ds = ancpbids.load_dataset(DS005_DIR)
         schema = ds.get_schema()
         file_paths = ds.select(schema.Artifact) \
             .where(all_of(eq(schema.Artifact.suffix, 'bold'),
-                          entity(schema, schema.EntityEnum._subject, '02'))) \
+                          entity(schema, schema.EntityEnum.subject, '02'))) \
             .get_file_paths()
         file_paths = list(file_paths)
         self.assertEqual(3, len(file_paths))
